{
module Example1 where
import Char
}

%name parser
%tokentype { Token }

%token 
	let		{ TokenLet }
	in		{ TokenIn }
	int		{ TokenInt $$ }
	var		{ TokenVar $$ }
	'='		{ TokenEq }
	'+'		{ TokenPlus }
	'-'		{ TokenMinus }
	'*'		{ TokenTimes }
	'/'		{ TokenDiv }
	'('		{ TokenOB }
	')'		{ TokenCB }


%%


Exp :: { S }
Exp  : Exp '+' Term		{ $1 :+: $3 }
     | Exp '-' Term		{ $1 :-: $3 }
     | Term			{ $1 }

Term : Term '*' Factor	        { $1 :*: $3 } 
     | Term '/' Factor	        { $1 :/: $3 }
     | Factor			{ $1 }

Factor : int			{ CI $1 }
       | var			{ V  $1 }
       | '(' Exp ')'		{ $2 }
       | let var '=' Exp in Exp	{ Let ($2,$4) $6 }

{


data Token
 	= TokenLet
 	| TokenIn
 	| TokenInt Integer
 	| TokenVar String
 	| TokenEq
 	| TokenPlus
 	| TokenMinus
 	| TokenTimes
 	| TokenDiv
 	| TokenOB
 	| TokenCB

happyError :: [Token] -> a
happyError _ = error ("Parse error\n")

data S = CI Integer
       | V String
       | Neg S
       | S :+: S
       | S :-: S
       | S :*: S
       | S :/: S
       | Let (String,S) S
       deriving (Eq,Show)
 

lexer :: String -> [Token]
lexer [] = []
lexer (c:cs) 
 	| isSpace c = lexer cs
	| isDigit c = lexNum (c:cs)
	| isAlpha c = lexVar (c:cs)
lexer ('=':cs) = TokenEq : lexer cs
lexer ('+':cs) = TokenPlus : lexer cs
lexer ('-':cs) = TokenMinus : lexer cs
lexer ('*':cs) = TokenTimes : lexer cs
lexer ('/':cs) = TokenDiv : lexer cs
lexer ('(':cs) = TokenOB : lexer cs
lexer (')':cs) = TokenCB : lexer cs

lexNum cs = TokenInt (read num) : lexer rest
	where (num,rest) = span isDigit cs

lexVar cs =
   case span isAlpha cs of
	("let",rest) -> TokenLet : lexer rest
	("in",rest)  -> TokenIn : lexer rest
	(var,rest)   -> TokenVar var : lexer rest

parse :: String -> S
parse = parser . lexer

}
