{
module Example2 where
import Char
}

%name parser
%tokentype { Token }

%token 
	let		{ TokenLet }
	in		{ TokenIn }
	int		{ TokenInt $$ }
	var		{ TokenVar $$ }
	'='		{ TokenEq }
	'+'		{ TokenPlus }
	'-'		{ TokenMinus }
	'*'		{ TokenTimes }
	'/'		{ TokenDiv }
	'('		{ TokenOB }
	')'		{ TokenCB }


%%


Exp :: { Env->Integer }
Exp  : Exp '+' Term		{ \p -> $1 p + $3 p }
     | Exp '-' Term		{ \p -> $1 p - $3 p }
     | Term			{ $1 }

Term : Term '*' Factor	        { \p -> $1 p * $3 p } 
     | Term '/' Factor	        { \p -> $1 p `div` $3 p}
     | Factor			{ $1 }

Factor : int			{ const $ $1 }
       | var			{ \p -> case lookup $1 p of
	     Nothing -> error ($1 ++ " not found")
             Just i -> i }
       | '(' Exp ')'		{ $2 }
| let var '=' Exp in Exp	{ \p -> $6 (($2,$4 p):p) }

{


data Token
 	= TokenLet
 	| TokenIn
 	| TokenInt Integer
 	| TokenVar String
 	| TokenEq
 	| TokenPlus
 	| TokenMinus
 	| TokenTimes
 	| TokenDiv
 	| TokenOB
 	| TokenCB

happyError :: [Token] -> a
happyError _ = error ("Parse error\n")

type Env = [(String,Integer)]
 

lexer :: String -> [Token]
lexer [] = []
lexer (c:cs) 
 	| isSpace c = lexer cs
	| isDigit c = lexNum (c:cs)
	| isAlpha c = lexVar (c:cs)
lexer ('=':cs) = TokenEq : lexer cs
lexer ('+':cs) = TokenPlus : lexer cs
lexer ('-':cs) = TokenMinus : lexer cs
lexer ('*':cs) = TokenTimes : lexer cs
lexer ('/':cs) = TokenDiv : lexer cs
lexer ('(':cs) = TokenOB : lexer cs
lexer (')':cs) = TokenCB : lexer cs

lexNum cs = TokenInt (read num) : lexer rest
	where (num,rest) = span isDigit cs

lexVar cs =
   case span isAlpha cs of
	("let",rest) -> TokenLet : lexer rest
	("in",rest)  -> TokenIn : lexer rest
	(var,rest)   -> TokenVar var : lexer rest

parse :: String -> Integer
parse s = let toks = lexer s
          in parser toks []

}
