module Main where

import IO
import Monad
import Parser

type Env = [(Var,Int)]

interpret :: Env -> Statement -> IO Env
interpret env x = 
 case x of
  Read v -> do
             putStr (v++": ")
             val <- getLine
             let r = read val :: Int
             return ((v,r):env)
  Write e -> do
              let v = eval env e
              putStr (show (v::Int) ++"\n")
              return env
  v ::=: e -> do
               let r = eval env e
               return ((v,r):env)
  For (v,l,h) stmts 
           -> foldM (\env' i -> foldM interpret ((v,i):env') stmts) 
                    env
                    [eval env l .. eval env h]

eval :: Env -> Exp -> Int
eval env (C i) = i
eval env (V v) = case lookup v env of
                   Nothing -> error (show v ++ " undefined")
                   Just i  -> i
eval env (a :+: b) = eval env a + eval env b
eval env (a :-: b) = eval env a - eval env b
eval env (a :*: b) = eval env a * eval env b


main :: IO ()
main = do
        hSetBuffering stdout NoBuffering
        putStr "Eingabedatei: "
        s <- getLine
        str <- readFile s
        let (declarations,statements) = parse str
	putStr (show statements)
        env <- foldM interpret [] statements
        putStr $ "\nEnvironment: "++ show env ++ "\n"
        return ()
