{
module Parser(Declaration,Statement(..),Exp(..),Var,parse) where

import Scanner

import Char
}

%name parser
%tokentype { Token }

%token  program         { Tprogram }
        read            { Tread }
        write           { Twrite }
        for             { Tfor }
        ":="            { Tassign }
        to              { Tto }
        do              { Tdo }
        begin           { Tbegin }
        end             { Tend }
 	int		{ Tint $$ }
	var		{ Tvar $$ }
	';'		{ Tsemicolon }
	'+'		{ Tplus }
	'-'		{ Tminus }
	'*'		{ Ttimes }
	'('		{ Tlparen }
	')'		{ Trparen }
        '.'             { Tdot } 

%left '+' '-'
%left '*'

%%

Prg :: { ([Declaration],[Statement]) }
Prg : program var ';' Stmt '.'  { ([],$4) }

Block :: { [Statement] }
Block : {- empty -}             { [] }
      | Stmt                    { $1 }
      | Block ';' Stmt          { $1 ++ $3 }

Stmt :: { [Statement] }
     : read '(' var ')'                { [Read $3] }
     | write '(' Exp ')'               { [Write $3] }
     | var ":=" Exp                    { [$1 ::=: $3] } 
     | for var ":=" Exp to Exp do Stmt { [For ($2,$4,$6) $8] }
     | begin Block end                 { $2 }
     | begin Block ';' end             { $2 }

Exp :: { Exp }
    : int         { C $1 }
    | var         { V $1 }
    | Exp '+' Exp { $1 :+: $3 }
    | Exp '-' Exp { $1 :-: $3 }
    | Exp '*' Exp { $1 :*: $3 }
    | '(' Exp ')' { $2 } 

{

happyError :: [Token] -> a
happyError _ = error ("Parse error\n")


type Var = String

type Declaration = ()

data Statement = Read Var
               | Write Exp
               | Var ::=: Exp
               | For (Var,Exp,Exp) [Statement]
               deriving Show
           
data Exp = C Int
         | V Var
         | Exp :+: Exp
         | Exp :-: Exp
         | Exp :*: Exp
         deriving Show



parse :: String -> ([Declaration],[Statement])
parse = parser . alexScanTokens

}
